@@ -1,5 +1,7 @@
 """Numpy-related utilities."""
 from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 
 import numpy as np
 
@@ -17,12 +19,18 @@ def to_categorical(y, num_classes=None):
     # Returns
         A binary matrix representation of the input.
     """
-    y = np.array(y, dtype='int').ravel()
+    y = np.array(y, dtype='int')
+    input_shape = y.shape
+    if input_shape and input_shape[-1] == 1 and len(input_shape) > 1:
+        input_shape = tuple(input_shape[:-1])
+    y = y.ravel()
     if not num_classes:
         num_classes = np.max(y) + 1
     n = y.shape[0]
     categorical = np.zeros((n, num_classes))
     categorical[np.arange(n), y] = 1
+    output_shape = input_shape + (num_classes,)
+    categorical = np.reshape(categorical, output_shape)
     return categorical
 
 
